/**
 * @file
 * Javascript for Tian Maps integration.
 */

(function ($, Drupal) {
  'use strict';

  /* global H */

  /**
   * @typedef {Object} TianMapSettings
   * @property tianMapsAppId
   * @property tianMapsAppCode
   */

  /**
   * GeolocationTianMap element.
   *
   * @constructor
   * @augments {GeolocationMapBase}
   * @implements {GeolocationMapInterface}
   * @inheritDoc
   *
   * @prop {Object} settings.tian_settings - Tian Maps specific settings.
   * @prop {T.Map} tianMap
   */
  function GeolocationTianMap(mapSettings) {
    if (typeof T === 'undefined') {
      console.error('Tian Maps library not loaded. Bailing out.'); // eslint-disable-line no-console.
      return;
    }

    this.type = 'tian';

    Drupal.geolocation.GeolocationMapBase.call(this, mapSettings);

    var defaultTianSettings = {
      zoom: 10
    };

    // Add any missing settings.
    this.settings.tian_settings = $.extend(defaultTianSettings, this.settings.tian_settings);

    // Set the container size.
    this.container.css({
      height: this.settings.tian_settings.height,
      width: this.settings.tian_settings.width
    });

    var that = this;

    // Instantiate (and display) a map object:
    this.tianMap = new T.Map(this.container.get(0));
    this.tianMap.centerAndZoom(new T.LngLat(0, 0), 10);

    // TODO: Centrering & Zooming.
    this.tianMap.enableScrollWheelZoom();

    this.tianMap.addEventListener('click', function (e) {
      that.clickCallback({lat: e.lnglat.lat, lng: e.lnglat.lng});
    });

    this.tianMap.addEventListener('moveend', function () {
      that.boundsChangedCallback(that.tianMap.getBounds());
    });

    this.initializedCallback();
    this.populatedCallback();
  }
  GeolocationTianMap.prototype = Object.create(Drupal.geolocation.GeolocationMapBase.prototype);
  GeolocationTianMap.prototype.constructor = GeolocationTianMap;
  GeolocationTianMap.prototype.getZoom = function () {
    var that = this;
    return new Promise(function (resolve, reject) {
      resolve(that.tianMap.getZoom());
    });
  };
  GeolocationTianMap.prototype.setZoom = function (zoom, defer) {
    if (typeof zoom === 'undefined') {
      zoom = this.settings.tian_settings.zoom;
    }
    zoom = parseInt(zoom);
    this.tianMap.setZoom(zoom);
  };
  GeolocationTianMap.prototype.setCenterByCoordinates = function (coordinates, accuracy, identifier) {
    Drupal.geolocation.GeolocationMapBase.prototype.setCenterByCoordinates.call(this, coordinates, accuracy, identifier);
    this.tianMap.centerAndZoom(new T.LngLat(coordinates.lng, coordinates.lat));
  };
  GeolocationTianMap.prototype.setMapMarker = function (markerSettings) {
    var tianMarkerSettings = {
      title: markerSettings.title
    };

    if (typeof markerSettings.icon === 'string') {
      tianMarkerSettings.icon = new T.Icon(markerSettings.icon, new T.Size(300,157));
    }
    var currentMarker = new T.Marker(new T.LngLat(parseFloat(markerSettings.position.lng), parseFloat(markerSettings.position.lat)), tianMarkerSettings);
    this.tianMap.addOverLay(currentMarker);

    currentMarker.locationWrapper = markerSettings.locationWrapper;

    Drupal.geolocation.GeolocationMapBase.prototype.setMapMarker.call(this, currentMarker);
    return currentMarker;
  };
  GeolocationTianMap.prototype.removeMapMarker = function (marker) {
    Drupal.geolocation.GeolocationMapBase.prototype.removeMapMarker.call(this, marker);
    this.tianMap.removeOverlay(marker);
  };
  GeolocationTianMap.prototype.addShape = function (shapeSettings) {
    if (typeof shapeSettings === 'undefined') {
      return;
    }

    var shape;

    switch (shapeSettings.shape) {
      case 'line':
        shape = new T.Polyline(shapeSettings.coordinates, {
          strokeColor: shapeSettings.strokeColor,
          strokeOpacity: shapeSettings.strokeOpacity,
          strokeWeight: shapeSettings.strokeWidth
        });
        break;

      case 'polygon':
        shape = new T.Polygon(shapeSettings.coordinates, {
          strokeOpacity: shapeSettings.strokeOpacity,
          strokeWeight: shapeSettings.strokeWidth,
          fillColor: shapeSettings.fillColor,
          fillOpacity: shapeSettings.fillOpacity
        });
        break;
    }

    this.tianMap.addOverLay(shape);
    Drupal.geolocation.GeolocationMapBase.prototype.addShape.call(this, shape);

    return shape;

  };
  GeolocationTianMap.prototype.removeShape = function (shape) {
    if (typeof shape === 'undefined') {
      return;
    }
    Drupal.geolocation.GeolocationMapBase.prototype.removeShape.call(this, shape);
    this.tianMap.removeOverlay(shape);
  };
  GeolocationTianMap.prototype.fitBoundaries = function (boundaries, identifier) {
    boundaries = this.denormalizeBoundaries(boundaries);
    if (!boundaries) {
      return;
    }

    if (!this.tianMap.getBounds().equals(boundaries)) {
      this.tianMap.setViewport([boundaries.getNorthEast(), boundaries.getSouthWest()]);
      Drupal.geolocation.GeolocationMapBase.prototype.fitBoundaries.call(this, boundaries, identifier);
    }
  };
  GeolocationTianMap.prototype.getMarkerBoundaries = function (locations) {

    locations = locations || this.mapMarkers;
    if (locations.length === 0) {
      return;
    }

    var points = [];
    $.each(
      locations,
        /**
         *
         * @param index
         * @param {T.Marker} marker
         */
      function (index, marker) {
        points.push(marker.getPosition());
      }
    );

    if (points.length === 0) {
      return;
    }

    var bounds = new T.Bounds(points[0], points[0]);

    $.each(
      points,
      function (index, point) {
        bounds.extend(point);
      }
    );

    return bounds;
  };
  GeolocationTianMap.prototype.getCenter = function () {
    var center = this.tianMap.getCenter();
    return {lat: center.lat, lng: center.lng};
  };
  GeolocationTianMap.prototype.normalizeBoundaries = function (boundaries) {
    if (boundaries instanceof T.Bounds) {
      return {
        north: boundaries.getNorthEast().lat,
        east: boundaries.getNorthEast().lng,
        south: boundaries.getSouthWest().lat,
        west: boundaries.getSouthWest().lng
      };
    }

    return false;
  };
  GeolocationTianMap.prototype.denormalizeBoundaries = function (boundaries) {
    if (typeof boundaries === 'undefined') {
      return false;
    }

    if (boundaries instanceof T.Bounds) {
      return boundaries;
    }

    if (Drupal.geolocation.GeolocationMapBase.prototype.boundariesNormalized.call(this, boundaries)) {
      return new T.Bounds(boundaries.north, boundaries.west, boundaries.south, boundaries.east);
    }
    else {
      boundaries = Drupal.geolocation.GeolocationMapBase.prototype.normalizeBoundaries.call(this, boundaries);
      if (boundaries) {
        return new T.Bounds(new T.LngLat(boundaries.north, boundaries.west), new T.LngLat(boundaries.south, boundaries.east));
      }
    }

    return false;
  };

  Drupal.geolocation.GeolocationTianMap = GeolocationTianMap;
  Drupal.geolocation.addMapProvider('tian', 'GeolocationTianMap');

})(jQuery, Drupal);
